@@ -116,7 +116,6 @@ class BandNames(ListEnum):
 
 
 # ---------------------- SAR ----------------------
-@unique
 class SarBandNames(BandNames):
     """SAR Band names"""
 
@@ -124,25 +123,37 @@ class SarBandNames(BandNames):
     """ Vertical Transmit-Vertical Receive Polarisation """
 
     VV_DSPK = "VV_DSPK"
-    """ Vertical Transmit-Vertical Receive Polarisation Despeckled """
+    """ Vertical Transmit-Vertical Receive Polarisation (Despeckled) """
 
     HH = "HH"
     """ Horizontal Transmit-Horizontal Receive Polarisation """
 
     HH_DSPK = "HH_DSPK"
-    """ Horizontal Transmit-Horizontal Receive Polarisation Despeckled """
+    """ Horizontal Transmit-Horizontal Receive Polarisation (Despeckled) """
 
     VH = "VH"
     """ Vertical Transmit-Horizontal Receive Polarisation """
 
     VH_DSPK = "VH_DSPK"
-    """ Vertical Transmit-Horizontal Receive Polarisatio Despeckled """
+    """ Vertical Transmit-Horizontal Receive Polarisation (Despeckled) """
 
     HV = "HV"
     """ Horizontal Transmit-Vertical Receive Polarisation """
 
     HV_DSPK = "HV_DSPK"
-    """ Horizontal Transmit-Vertical Receive Polarisation Despeckled """
+    """ Horizontal Transmit-Vertical Receive Polarisation (Despeckled) """
+
+    RH = "RH"
+    """ Compact polarization: right circular transmit, horizontal receive """
+
+    RH_DSPK = "RH_DSPK"
+    """ Compact polarization: right circular transmit, horizontal receive """
+
+    RV = "RV"
+    """ Compact polarization: right circular transmit, vertical receive (Despeckled) """
+
+    RV_DSPK = "RV_DSPK"
+    """ Compact polarization: right circular transmit, horizontal receive (Despeckled) """
 
     @classmethod
     def corresponding_despeckle(cls, band: "SarBandNames"):
@@ -255,6 +266,9 @@ class OpticalBandNames(BandNames):
     GREEN = "GREEN"
     """Green"""
 
+    YELLOW = "YELLOW"
+    """Yellow"""
+
     RED = "RED"
     """Red"""
 
@@ -276,9 +290,6 @@ class OpticalBandNames(BandNames):
     WV = "WATER_VAPOUR"
     """Water vapour"""
 
-    FAR_NIR = "FAR_NIR"
-    """Far NIR"""
-
     SWIR_CIRRUS = "CIRRUS"
     """Cirrus"""
 
@@ -288,9 +299,6 @@ class OpticalBandNames(BandNames):
     SWIR_2 = "SWIR_2"
     """SWIR, Band 2"""
 
-    MIR = "MIR"
-    """MIR"""
-
     TIR_1 = "THERMAL_IR_1"
     """Thermal IR, Band 1"""
 
@@ -300,6 +308,78 @@ class OpticalBandNames(BandNames):
     PAN = "PANCHROMATIC"
     """Panchromatic"""
 
+    # SLSTR additional band names
+    S7 = "S7"
+    """
+    S7
+    """
+
+    F1 = "F1"
+    """
+    F1
+    """
+
+    F2 = "F2"
+    """
+    F2
+    """
+
+    # OLCI additional band names
+    Oa01 = "Oa01"
+    """
+    Oa01
+    """
+
+    Oa02 = "Oa02"
+    """
+    Oa02
+    """
+
+    Oa05 = "Oa05"
+    """
+    Oa05
+    """
+
+    Oa09 = "Oa09"
+    """
+    Oa09
+    """
+
+    Oa10 = "Oa10"
+    """
+    Oa10
+    """
+
+    Oa13 = "Oa13"
+    """
+    Oa13
+    """
+
+    Oa14 = "Oa14"
+    """
+    Oa14
+    """
+
+    Oa15 = "Oa15"
+    """
+    Oa15
+    """
+
+    Oa18 = "Oa18"
+    """
+    Oa18
+    """
+
+    Oa19 = "Oa19"
+    """
+    Oa01
+    """
+
+    Oa21 = "Oa21"
+    """
+    Oa01
+    """
+
 
 # too many ancestors
 # pylint: disable=R0901
